#!/usr/bin/python
# -*- coding: utf-8 -*-

##
#	Prosty kod obrabiajacy dane GIS. Dokonuje zakodowania
#	do formatu "Encoded Polylines" uzywanego przez Google Maps.
#
#	Przyklady dostepny na stronie Google Maps Api:
#	http://www.google.com/apis/maps/documentation/#Encoded_Polylines
#
#	Kod w znacznej czesci przepisany z JavaScriptu na Python.
#	Orginal na stronie (zrodla zawieraja odwolanie do pliku .js):
#	http://facstaff.unca.edu/mcmcclur/GoogleMaps/EncodePolyline/
#
#	\author Krzysztof Kosyl


from math import floor,asin,sin,cos
import sys

##
#	Przeksztalca liste punktow do formatu Google Encoded Polyline
def encodePolyline(polyline):
	def encodeNumber(num):
		result = ""
		while (num >= 0x20):
			result += chr(0x5f + num % 0x20)
			num /= 0x20
		return result + chr(0x3f + num % 0x20)
		
	def encodeSignedNumber( num):
		num = 2 * num
		if (num < 0):
			num =  -num - 1
		return encodeNumber(num)
		
	def dist(p1, p2):
		deg = 0.0174532925199
		
		def foo(x1, x2):
			return sin((x1 - x2)*deg/2)**2
		
		return 12746004.5 * asin((foo(p1[0], p2[0]) + cos(p1[0] * deg) * cos(p2[0] * deg) * foo(p1[1], p2[1])) ** 0.5)
	
	def setLevels(points):
		"""
		Przydziela punktom "poziom szczegolowosci" do ktorego naleza.
		Im ta liczba jest wieksza tym punkt jest wazniejszy. W zaleznosci
		od przyblizenia beda uwzgledniane coraz mniej warzne punkty.
		"""
		
		numLevels = 18
		small0 = 500000
	
		points[0][2] = numLevels-1;
		points[-1][2] = numLevels-1;
		
		for level in range(numLevels-1, -1, -1):
			small = small0 * 2.0 ** (level-18)
			
			i, j = 1, 1
			while i < len(points)-1:
				if dist(points[i], points[j]) >= small and points[i][2] == None: 
					points[i][2] = level;
					j = i;
				if points[i][2] != None:
					j = i
				i += 1
				
		for i in points: 
			if i[2] == None:
				i[2] = 0
				
	def createEncodings(points):
		prev_lat = 0
		prev_lng = 0
		
		setLevels(points)
		
		encoded_points = ''
		encoded_levels = ''
		for point in points:
			current_lat = int(floor(point[0] * 10 ** 5))
			current_lng = int(floor(point[1] * 10 ** 5))
			prev_lat, delta_lat = current_lat, current_lat - prev_lat
			prev_lng, delta_lng = current_lng, current_lng - prev_lng
			encoded_points += encodeSignedNumber(delta_lat) + encodeSignedNumber(delta_lng)
			encoded_levels += encodeNumber(point[2])
		return (encoded_points, encoded_levels)
	
	
	points = [[a, b, None] for (a,b) in zip(polyline[0::2], polyline[1::2])]
	return createEncodings(points)

