# -*- coding: utf-8 -*-

##
#	Obsluga RSS i Atom
#
#	\author Krzysztof Kosyl

from django.contrib.syndication.feeds import Feed
from django.utils.feedgenerator import Atom1Feed
from django.core.exceptions import ObjectDoesNotExist
from django.conf import settings 

from django.utils.html import escape

from nebuchadrezzar.semiramis.models import Track, TrackComment, Mark, MarkComment, MarkPhoto
from nebuchadrezzar.semiramis.templatetags.semiramis_templatetags import link_to, global_link


class TrackFeedRss(Feed):
	title = _(u"Semiramis tracks news")
	
	link = global_link("/track/")
	description = _(u"Updates on tracks additions on Semiramis")
	
	description_template = 'feeds/common_description.html'
	
	def item_link(self, item):
		return global_link(item)
	def item_pubdate(self, item):
		return item.date
	def items(self):
		return Track.objects.order_by('-date')[:10]
	
class TrackFeedAtom(TrackFeedRss):
    feed_type = Atom1Feed
    subtitle = TrackFeedRss.description
	
	
	
	
	
class MarkFeedRss(Feed):
	title = _(u"Semiramis marks news")
	link = global_link("/mark/")
	description = _(u"Updates on marks additions on Semiramis")
	
	description_template = 'feeds/common_description.html'
	
	def item_link(self, item):
		return global_link(item)
	def item_pubdate(self, item):
		return item.date
	def items(self):
		return Mark.objects.order_by('-date')[:10]
	
class MarkFeedAtom(MarkFeedRss):
    feed_type = Atom1Feed
    subtitle = MarkFeedRss.description
	
	
	
	
	
class CommentFeedRss(Feed):
	def title(self, obj):
		if obj[0] == 'track':
			return _(u"Semiramis: new comments for track '%(name)s'") % {'name':unicode(obj[1].name, 'utf-8')}
		else:
			return _(u"Semiramis: new comments for mark '%(name)s'") % {'name':unicode(obj[1].name, 'utf-8')}
		
	def link(self, obj):
		return global_link(obj)
	
	def description(self, obj):
		if obj == 'track':
			return _(u"Last comments for track '%(name)s'") % {'name':unicode(obj[1].name, 'utf-8')}
		else:
			return _(u"Last comments for mark '%(name)s'") % {'name':unicode(obj[1].name, 'utf-8')}
	
	description_template = 'feeds/common_description.html'
	title_template = 'feeds/truncate_title.html'
	
	def get_object(self, bits):
		try:
			i = int(bits[1])
			
			if bits[0] == 'track':
				track = Track.objects.get(id=i)
				return ('track', track, track.trackcomment_set.all()[:10])
			elif bits[0] == 'mark':
				mark = Mark.objects.get(id=i)
				return ('mark', mark, mark.markcomment_set.all()[:10])
			else:
				raise ObjectDoesNotExist
		except:
			raise ObjectDoesNotExist
	
	def item_link(self, item):
		return global_link(item)
	def item_pubdate(self, item):
		return item.date
	def items(self, obj):
		return obj[2]
	
class CommentFeedAtom(CommentFeedRss):
    feed_type = Atom1Feed
    subtitle = CommentFeedRss.description


class PhotoFeedRss(Feed):
	def title(self, obj):
		return _(u"Semiramis: new photos for mark '%(name)s'") % {'name':unicode(obj.name, 'utf-8')}
		
	def link(self, obj):
		return global_link(obj)
	
	def description(self, obj):
		return _(u"Last photos for mark '%(name)s'") % {'name':unicode(obj.name, 'utf-8')}
	
	description_template = 'feeds/photo_description.html'
	#title_template = 'feeds/truncate_title.html'
	
	def get_object(self, bits):
		try:
			i = int(bits[0])
			return  Mark.objects.get(id=i)
		except:
			raise ObjectDoesNotExist

	
	def item_link(self, item):
		return global_link(item)
	def item_pubdate(self, item):
		return item.date
	def items(self, obj):
		print obj, obj.markphoto_set.all()
		return obj.markphoto_set.all()[:10]


class PhotoFeedAtom(PhotoFeedRss):
    feed_type = Atom1Feed
    subtitle = TrackFeedRss.description

