from django.conf import settings
import socket

##
#	Plik zawierajacy klase udostepniajaca lacze do modulu Hammurabi.
#
#	\author Krzysztof Kosyl


## Klasa udostepniajaca lacze do modulu Hammurabi.
#
class HammurabiLink:
	
	##	
	#	Konstruktor klasy.
	#	
	#	Pobiera adres TCP/IP na ktorym nasluchuje Hammurabi.
	#	Jesli ktorys z parametrow nie jest podany wykorzystywane 
	#	sa ustawienia podane w pliku 'settings.py'
	#	\param host adres maszyny 
	#	\param port port docelowy
	def __init__(self, host=None, port=None):
		if host == None:
			self.host = settings.SEMIRAMIS_HAMMURABI_HOST
		else:
			self.host = host
		if port == None:
			self.port = settings.SEMIRAMIS_HAMMURABI_PORT
		else:
			self.port = port
		
		
	##	
	#	Dokonuje tranzakcji z Hammurabim.
	#
	#	Wykonuje sie sekwencyjnie:
	#	- laczy sie z Hammurabim
	#	- wysyla zapytanie w calosci
	#	- zamyka socket "do zapisu"
	#	- odbiera wszystkie dane z soketu
	#	- zamyka socket
	def socket_get(self, request, size = 4096):
		s = socket.socket(socket.AF_INET,socket.SOCK_STREAM)
		s.connect((self.host, self.port))
		s.sendall(request)
		s.shutdown(socket.SHUT_WR)
		buf = s.recv(size)
		result = []
		while len(buf) > 0:
			try:
				result.append(buf)
				buf = s.recv(size)
			except:
				pass
		s.close()
		result = ''.join(result)
		return result
		
		
	##	
	#	Laduje plik wczytany z komorki do Hamurabiego.
	#
	#	Wysyla dane pobrane od uzytkownika i przesyla je do Hammurabiego
	# 	\param data dane od uzytkownika w postaci napisu
	#	\return krotka ktorej pierwszym elementem jest lista numerow sciezek
	#		a drugim lista krotek postaci (lat, lng, kind, name)
	def upload(self, data):
		tracks = []
		marks = []
		
		buf = "upload\n" + data
		result = self.socket_get(buf)
		result = result.split("\n")
		for line in result:
			try:
				if line[0] != '#':
					tracks.append(int(line))
				else:
					items = line[1:].split(':', 3)
					marks.append((
							float(items[0]),
							float(items[1]),
							int(items[2]),
							items[3]
						))
			except:
				pass
		return (tracks, marks)
				
	## 
	#	
	def delete_track(i):
		self.socket_get("delete\n%d\n" % i)
	
	##	
	#	Wysyla zapytanie do Hammurabiego, zwraca jego wynik
	#
	#	Podaje argumenty z slownika ktory jest jej argumentem.
	#	
	#	\return zwraca liste krotek. W kazdej krotce pokolei jest key, kind 
	#		i lista wspolrzednych
	def query(self, **k):
		latA = k.pop('latA',  -90.0)
		lngA = k.pop('lngA', -180.0)
		latB = k.pop('latB',   90.0)
		lngB = k.pop('lngB',  180.0)
		
		q = "track\nlatA:%.6f\nlngA:%.6f\nlatB:%.6f\nlngB:%.6f\n" % (latA, lngA, latB, lngB)
		
		for a in k.items():
			q += '%s:%s\n' % a
			
		data = self.socket_get(q)
		
		def parse_points(s):
			return map(float, s.split())
		
		result = []
		for line in data.split('\n'):
			part = line.split(':')
			try:
				i, t = int(part[0]), max(int(part[1]), 0)
				result.append((i, t, parse_points(part[2])))
			except:
				pass
		return result

