# -*- coding: utf-8 -*-

##
#	Plik zawierajacy model bazy danych.
#
#	\author Krzysztof Kosyl

from django.db import models
from django.contrib.auth.models import User

from hammurabi_link import HammurabiLink

## Klasa opisyjaca tabele zawierajaca sciezki.
class Track(models.Model):
	## Identyfikator uzywany przez Hammurabiego
	key = models.IntegerField(unique=True, blank=False, null=False)
	## Uzytkownik ktory dodal sciezkie
	user = models.ForeignKey(User)
	## Nazwa sciezki
	name = models.CharField(blank=False, maxlength=100)
	## Opis sciezkli
	text = models.TextField(blank=True)
	## Data dodania sciezki
	date = models.DateTimeField(auto_now_add = True)
	## Kolor jakim sciezka ma byc oznaczona
	color = models.CharField(blank=True, default='', maxlength=3)
	
	def delete(self):
		i = self.id
		super(Track, self).delete()
		#try:
			#link = HammurabiLink()
			#link.delete_track(i)
		#except:
			#pass
	
	def __str__(self):
		return self.name

	class Meta:
		ordering = ('-date',)

	class Admin:
		pass	


## Klasa opisyjaca tabele zawierajaca komantarze do sciezek.
class TrackComment(models.Model):
	## Klucz obcy do sciezki dla ktorej jest to komentarz
	track = models.ForeignKey(Track)
	## Uzytkownik ktory dodal komentarz
	user = models.ForeignKey(User)
	## Tresc komentarza
	text = models.TextField(blank=False)
	## Data dodania komentarza
	date = models.DateTimeField(auto_now_add = True)

	def __str__(self):
		return self.text

	class Meta:
		ordering = ('date',)

	class Admin:
		pass


## Klasa opisyjaca tabele zawierajaca znaczniki.
class Mark(models.Model):
	## Uzytkownik ktory dodal znacznik
	user = models.ForeignKey(User)
	## Nazwa znacznika
	name = models.CharField(blank=False, maxlength=100)
	## Opis znacznika
	text = models.TextField(blank=True)
	## Data dodania znacznika
	date = models.DateTimeField(auto_now_add = True)
	## Rodzaj znacznika (numer odpowiadajacy rodzajowi)
	kind = models.IntegerField(default=0)
	## Tytul znacznika (uzywane wylacznie przez Hammurabiego)
	title = models.CharField(blank=False, maxlength=100, default='')
	## Poziom waznosci znacznika - im wiekszy tym przy mniejszym zoomie widac
	#  znacznik  
	severity = models.IntegerField(default=0)
	## Szerokosc geograficzna znacznika
	lat = models.FloatField(max_digits=10, decimal_places=6)
	## Dlugosc geograficzna znacznika
	lng = models.FloatField(max_digits=10, decimal_places=6)

	def __str__(self):
		return self.name

	class Meta:
		ordering = ('-date',)

	class Admin:
		pass	


## Klasa opisyjaca tabele zawierajaca komentarze do znacznikow.
class MarkComment(models.Model):
	## Klucz obcy do znacznika dla ktorego jest to komentarz
	mark = models.ForeignKey(Mark)
	## Uzytkownik ktory dodal komentarz
	user = models.ForeignKey(User)
	## Tresc komentarza
	text = models.TextField(blank=False)
	## Data dodania komentarza
	date = models.DateTimeField(auto_now_add = True)

	def __str__(self):
		return self.text

	class Meta:
		ordering = ('date',)

	class Admin:
		pass


## Klasa opisyjaca tabele zawierajaca informacje o zdjeciach.
class MarkPhoto(models.Model):
	## Klucz obcy do znacznika dla ktorego jest to zdjecie.
	mark = models.ForeignKey(Mark)
	## Uzytkownik ktory dodal zdjecie
	user = models.ForeignKey(User)
	## Nazwa zdjecia 
	name = models.TextField(blank=False)
	## Data dodania zdjecia
	date = models.DateTimeField(auto_now_add = True)

	def __str__(self):
		return self.name

	class Meta:
		ordering = ('date',)

	class Admin:
		pass

