# -*- coding: utf-8 -*-

##
#	Plik zawierajacy funkcje pomocnicze dla szablonow, tzn filtry 
#	i znaczniki szablonowe.
#
#	\author Krzysztof Kosyl

from django import template
from django.conf import settings

from nebuchadrezzar.semiramis.models import Track, TrackComment, Mark, MarkComment, MarkPhoto
from django.contrib.auth.models import User

from django.utils.html import escape, linebreaks



register = template.Library()

##
#	Tag tworzacy liste stron
def pager(context):
	data = context['pager']
	
	return data					
	#page_numbers = range(max(0, context['page']-adjacent_pages), min(context['pages'], context['page']+adjacent_pages)+1)
	return {
		"hits": data.hits,
		"results_per_page": data.num_per_page,
		"page": data.page,
		"pages": data.pages,
		"page_numbers": page_numbers,
		"next": data.page+1,
		"previous": data.page-1,
		"has_next": data.has_next_page(data.page),
		"has_previous": data.has_previous_page(data.page),
		"show_first": 1 not in page_numbers,
		"show_last": data.pages not in page_numbers,
	}

register.inclusion_tag("pager.html", takes_context=True)(pager)

##
#	Formatowanie tekstu opisow, uzywajace jednego z dostepnych 
#	'markup language'
@register.filter(name='babel')
def babel(text, arg =''):
	try:
		import markdown
		max_len = 200
		if arg == 'head' and len(text) > max_len:
			text = text[:max_len] + '[...]'
		return markdown.markdown(text, safe_mode=True)
	except:
		return linebreaks(escape(value))


##
#	Formatowanie daty wedlug ustawien globalnych
@register.filter(name='date_format')
def date_format(item):
	try:
		if settings.SEMIRAMIS_DATE_FORMAT:
			format = settings.SEMIRAMIS_DATE_FORMAT
		else:
			format = '%m-%d-%Y %H:%M:%S'
		return item.strftime(format)
	except:
		return ''
	
##	
#	Funkcja mapujaca obiekty na ich adresy URL.
#	
#	Funkcja urzywana glownie w szablonach (lecz nie tylko).
#	Jej celem jest odzielenie  
@register.filter(name='global_link')
def global_link(item, arg=""):
	return settings.SEMIRAMIS_HOST_PREFIX + link_to(item, arg)

@register.filter(name='link_to')
def link_to(item, arg=""):
	return settings.SEMIRAMIS_DIR_PREFIX + do_link_to(item, arg)

def do_link_to(item, arg=""):
	try:
		result = ''
		if str(item) == item:
			if item[0] == '/':
				return item
			
			if item == 'rss:track':
				return '/rss/track/'
			if item == 'atom:track':
				return '/atom/track/'
			if item == 'rss:mark':
				return '/rss/mark/'
			if item == 'atom:mark':
				return '/atom/mark/'
			
			if item == 'main':
				return '/'
			if item == 'map':
				return '/map/'
			return '/%s/' % item
		if isinstance(item, Track):
			if arg == 'rss:comment':
				return '/rss/comment/track/%d/' % item.id
			if arg == 'atom:comment':
				return '/atom/comment/track/%d/' % item.id
			if arg == "dir":
				return "/track/"
			if arg == "icon":
				return "/static/marker/track.png"
			result = '/track/%d/' % item.id
			if arg in ('edit', 'delete'):
				result += arg + "/"
		elif isinstance(item, TrackComment):
			if arg == "dir":
				return link_to(item.track)
			if arg in ('edit', 'delete'):
				result += '/track/comment/%d/%s' % (item.id, arg)
			else:
				result = '/track/%d/#comment%d' % (item.track.id, item.id)
		elif isinstance(item, Mark):
			if arg == 'rss:comment':
				return '/rss/comment/mark/%d/' % item.id
			if arg == 'atom:comment':
				return '/atom/comment/mark/%d/' % item.id
			
			if arg == 'rss:photo':
				return '/rss/photo/%d/' % item.id
			if arg == 'atom:photo':
				return '/atom/photo/%d/' % item.id
			
			if arg == "dir":
				return "/mark/"
			if arg == "icon":
				return "/static/marker/icon%don.png" % item.kind
			result = '/mark/%d/' % item.id
			if arg in ('edit', 'delete'):
				result += arg + "/"
		elif isinstance(item, MarkComment):
			if arg == "dir":
				return link_to(item.mark)
			if arg in ('edit', 'delete'):
				result += '/mark/comment/%d/%s' % (item.id, arg)
			else:
				result += '/mark/%d/#comment%d' % (item.mark.id, item.id)
		elif isinstance(item, MarkPhoto):
			if arg == "crop":
				return '/media/photo/%.4d/img%.6d_s.jpg' % (item.id / 100, item.id)
			if arg == "thumb":
				return '/media/photo/%.4d/img%.6d_t.jpg' % (item.id / 100, item.id)
			if arg == "normal":
				return '/media/photo/%.4d/img%.6d_l.jpg' % (item.id / 100, item.id)
			if arg == "orginal":
				return '/media/photo/%.4d/img%.6d_o.jpg' % (item.id / 100, item.id)
			if arg == "dir":
				return link_to(item.mark)
			return '/photo/%d/' % item.id
			if arg in ('edit', 'delete'):
				result += arg + "/"
		elif isinstance(item, User):
			return '/user/%s/' % item.username
		else:
			return '/'
		return result 
	except:
		return '/'
		
