# -*- coding: utf-8 -*-

##
#	Plik zawierajacy mapowanie URL na widok.
#
#	\author Krzysztof Kosyl


from django.conf.urls.defaults import *
from django.contrib.auth.views import login, logout
from django.views.static import serve
from django.conf import settings 

from nebuchadrezzar.semiramis.feeds import TrackFeedRss, TrackFeedAtom
from nebuchadrezzar.semiramis.feeds import MarkFeedRss, MarkFeedAtom
from nebuchadrezzar.semiramis.feeds import CommentFeedRss, CommentFeedAtom
from nebuchadrezzar.semiramis.feeds import PhotoFeedRss, PhotoFeedAtom


urlpatterns = patterns('nebuchadrezzar.semiramis',
	(r'^$',									'views.main'),
	(r'^map/$',								'views.gmap'),
	
	(r'^track/$',							'views.track_page'),
	(r'^track/page([0-9]+)/$',				'views.track_page'),
	(r'^track/([0-9]+)/$',					'views.track_id'),
	(r'^track/([0-9]+)/edit/$',				'views.track_id_edit'),
	(r'^track/([0-9]+)/delete/$',			'views.track_id_delete'),
	(r'^track/comment/([0-9]+)/edit/$',		'views.track_comment_id_edit'),
	(r'^track/comment/([0-9]+)/delete/$',	'views.track_comment_id_delete'),
	
	(r'^mark/add/$',						'views.mark_add'),
	(r'^mark/$',							'views.mark_page'),
	(r'^mark/page([0-9]+)/$',				'views.mark_page'),
	(r'^mark/([0-9]+)/$',					'views.mark_id'),
	(r'^mark/([0-9]+)/edit/$',				'views.mark_id_edit'),
	(r'^mark/([0-9]+)/delete/$',			'views.mark_id_delete'),
	(r'^mark/comment/([0-9]+)/edit/$',		'views.mark_comment_id_edit'),
	(r'^mark/comment/([0-9]+)/delete/$',	'views.mark_comment_id_delete'),
	
	(r'^user/([a-zA-z0-9_]+)/$',				'views.user_login'),
	(r'^user/([a-zA-z0-9_]+)/photos/$',		'views.user_login_photos'),
	(r'^user/([a-zA-z0-9_]+)/tracks/$',		'views.user_login_tracks'),
	(r'^user/([a-zA-z0-9_]+)/marks/$',		'views.user_login_marks'),
	
	
	(r'^upload/$',							'views.upload'),
	(r'^upload/result/$',					'views.upload_result'),
	
	(r'^photo/([0-9]+)/$',					'views.photo_id'),
	(r'^photo/([0-9]+)/edit/$',				'views.photo_id_edit'),
	(r'^photo/([0-9]+)/delete/$',			'views.photo_id_delete'),
	
	(r'^accounts/profile/$',				'views.accounts_profile'),
	(r'^accounts/register/$',				'views.accounts_register'),
	(r'^accounts/created/$',				'views.accounts_created'),
	
	(r'^data/tracks/$',						'views.data_tracks'),
	(r'^data/marks/$',						'views.data_marks'),
	(r'^data/mark/$',						'views.data_mark'),
	(r'^data/babel/$',						'views.data_babel'),
	(r'^data/export/(.+)/export.([a-z]+)$',	'views.data_export'),
	
	(r'^sandbox/(.*)$',						'views.sandbox'),
	
	(r'^xmlrpc/$',							'xmlrpc.xmlrpc'),

)
		
feeds_rss = {
	'track':	TrackFeedRss,
	'mark':		MarkFeedRss,
	'comment':	CommentFeedRss,
	'photo':	PhotoFeedRss,
}

feeds_atom = {
	'track':	TrackFeedAtom,
	'mark':		MarkFeedAtom,
	'comment':	CommentFeedAtom,
	'photo':	PhotoFeedAtom,
}
		
urlpatterns += patterns('',
	(r'^accounts/login/$',					login, {'template_name':'accounts/login.html'}),
	(r'^accounts/logout/$',					logout, {'next_page':'/', 'template_name':'accounts/logout.html'}),
	(r'^static/(?P<path>.*)$',				serve, {'document_root': settings.SEMIRAMIS_STATIC_DIR}),
	(r'^media/(?P<path>.*)$',				serve, {'document_root': settings.SEMIRAMIS_MEDIA_DIR}),
	
	(r'^rss/(?P<url>.*)/$',					'django.contrib.syndication.views.feed', {'feed_dict': feeds_rss}),
	(r'^atom/(?P<url>.*)/$',				'django.contrib.syndication.views.feed', {'feed_dict': feeds_atom}),
)

