# -*- coding: utf-8 -*-

##
# 	Plik zawiera szereg funkcje pomocniczych
#
#	\author Krzysztof Kosyl



from django.http import HttpResponse
from django.http import HttpResponseRedirect, HttpResponseForbidden
from django.http import Http404
from django.conf import settings
from django.db.models.manager import Manager
from django import oldforms as forms
from django import newforms
#from django.newforms import form_for_model, form_for_instance, save_instance, BaseForm, Form, CharField

from django.shortcuts import render_to_response, get_object_or_404
from django.contrib.auth.decorators import login_required

from django.template import Context, loader

from django.contrib.auth.models import User

from django.contrib.auth.forms import UserCreationForm  #??
from django.core.paginator import ObjectPaginator, InvalidPage

from django.contrib.syndication.feeds import Feed
from django.utils.feedgenerator import Atom1Feed

from datetime import datetime
import os
from PIL import Image
import zipfile
try:
	import cStringIO as StringIO
except ImportError:
	import StringIO

from nebuchadrezzar.semiramis.models import Track, TrackComment, Mark, MarkComment, MarkPhoto
from nebuchadrezzar.semiramis.templatetags.semiramis_templatetags import link_to

from hammurabi_link import HammurabiLink
import socket


##	Laczy dane z request.POST i request.FILES.
#		
#	Polaczone dane sa wymagane podczas korzystania z uploadu
#	plikow w newforms.
#	\param request obiekt zapytania
def post_data(request):
	data = request.POST.copy()
	data.update(request.FILES)
	return data


def common(request, dict_):
	result =  dict(
			user=request.user,
			last_tracks=Track.objects.order_by('-date')[:10],
			last_marks=Mark.objects.order_by('-date')[:10],
			last_photos=MarkPhoto.objects.order_by('-date')[:10]
		)
	result.update(dict_)
	return result
	
##	Funkcja ulatwiajaca kasowanie obiektow.
#	
#	Funkacja wzorowana na get_object_or_404(). Sprawdza czy aktualny
#	uzytkownik ma prawo do skasowania obiektu. 
#	Jesli nie to zwraca obiekt klasy HttpResponseForbidden (odpowiadajacy
#	bledowi HTTP Error 403 Forbidden), wraz z zaladowanym szablonem.
#	W przeciwnym wypadku sprawdza jakia metoda HTTP zostala zastosowana.
#	Jesli jest to 'POST' to bez pytania kasuje obiekt i przechodzi do 
#	strony ktora go zawierala.
#	Dla innych metod zapytan wyswietla szablon pytajacy czy napewno 
#	skasowac obiekt (szablon zawiera rowniez lacze kierujace uzytkownika 
#	do metody POST, ktora skasuje obiekt).
#	
#	\param klass klasa modelu, z ktorego chcemy skasowac obiekt
#	\param request obiekt zapytania HTTP
#	\param args parametry zapytania do ORM
#	\param kwargs parametry zapytania do ORM
# 	\return obiekt klasy HttpResponse lub pochodnej od niej
def delete_object_or_forbidden(klass, request, *args, **kwargs):
	if isinstance(klass, Manager):
		manager = klass
		klass = manager.model
	else:
		manager = klass._default_manager
	
	try:
	
		item =  manager.get(*args, **kwargs)
		
		if not (request.user.is_staff or (item.user == request.user)):
			template = loader.get_template('new/forbidden.html')
			context = Context({'user': request.user})
			return HttpResponseForbidden(template.render(context))
		
		if request.method == 'POST':
			redirect = link_to(item, "dir")
			item.delete()
			return HttpResponseRedirect(redirect)
		else:
			return render_to_response('new/delete.html', {
					'user':request.user, 
					'item':item
			})

	except:
		raise Http404()
				
			
##	Funkcja ulatwiajaca edycje obiektow.
#	
#	Funkacja wzorowana na get_object_or_404(). Sprawdza czy aktualny
#	uzytkownik ma prawo do edycji obiektu. 
#	Jesli nie to zwraca obiekt klasy HttpResponseForbidden (odpowiadajacy
#	bledowi HTTP Error 403 Forbidden), wraz z zaladowanym szablonem.
#	W przeciwnym wypadku sprawdza jakia metoda HTTP zostala zastosowana.
#	Jesli jest to 'POST' to bez pytania modyfikuje obiekt, zapisuje go w bazie
#	danych  i przechodzi do jego strony.
#	Dla innych metod zapytan wyswietla szablon zawierajacy forme klasy 'form' 
#	zaladowana danymi z obiektu. (szablon zawiera rowniez lacze kierujace 
#	uzytkownika do metody POST, ktora zapisuje modyfikacje obiektu).
#	
#	\param klass klasa modelu, z ktorego chcemy skasowac obiekt
#	\param form klasa formatki edytujacej
#	\param editor funkcja dwuargumentowa przyjmujaca za parametry 
#		obiekt do zmodyfikowania i slownik zmian, zwracajaca zmodyfikowany
#		obiekt. Jesli None to automatycznie przepsuje zmiany korzystajac
#		z analogii nazw elemetów modelu i formatki. 
#	\param request obiekt zapytania HTTP
#	\param args parametry zapytania do ORM
#	\param kwargs parametry zapytania do ORM
# 	\return obiekt klasy HttpResponse lub pochodnej od niej	
def edit_object_or_forbidden(klass, form, editor, request, *args, **kwargs):
	
	if isinstance(klass, Manager):
		manager = klass
		klass = manager.model
	else:
		manager = klass._default_manager
	
	try:
		item =  manager.get(*args, **kwargs)
		if not (request.user.is_staff or (item.user == request.user)):
			template = loader.get_template('new/forbidden.html')
			context = Context({'user': request.user})
			return HttpResponseForbidden(template.render(context))
		if request.method == 'POST':
			edit_form = form(request.POST)
			if edit_form.is_valid():
				if editor != None:
					item = editor(item, edit_form.clean_data)
				else:
					for i in edit_form.clean_data:
						setattr(item, i, edit_form.clean_data[i])
				item.save()
			return HttpResponseRedirect(link_to(item))
		else:
			edit_form = form(item.__dict__)
			return render_to_response('new/edit.html', {
					'user':request.user, 
					'edit_form':edit_form,
					'item':item
				})

	except:
		raise Http404()


##	Zapisuje zdjecie na dysku i w bazie, tworzy rownierz miniaturki.
# 
#	Zapisuje zdjecie na dysk w postaci 4 plikow:
#	- square  - nazwa konczaca sie na  "_s" rozmiar okreslony w
#		SEMIRAMIS_PHOTO_THUMB, obrazek jest tak przycinany i skalowany,
#		ze powstaje obrazek o zadanych proporcjach 
#	- thumb   - nazwa konczaca sie na  "_t" skaluje obrazek tak aby powstal
#		obrazek mieszczacy sie w rozmiarze SEMIRAMIS_PHOTO_THUMB
#	- large   - nazwa konczaca sie na  "_l" skaluje obrazek tak aby powstal
#		obrazek mieszczacy sie w rozmiarze SEMIRAMIS_PHOTO_LARGE
#	- orginal - nazwa konczaca sie na  "_o" - rozmiar orginalny, nie jest 
#		skalowany i jesli jest to plik JPEG to nawet nie jest rekompresowany
#	Wszystkie pliki zapisywane sa w formacie JPEG, z dbaloscia o wysoka jakosc
#	skalowan i kompresji podczas zapisu
#
#	Zapisuje rownierz w bazie danych obiekt klasy 
#
#	\param mark obiekt klasy Mark do ktorego nalezy zdjecie
#	\param user uzytkownik dodajacy zdjecie
#	\param text opis zdjecia
#	\param data binarny bufor zawierajacy zawartosc pliku
def save_photo(mark, user, text, data):
	#try:
		#import cStringIO as StringIO
	#except ImportError:
		#import StringIO
	
	
	def mkdir(newdir):
		if os.path.isdir(newdir):
			pass
		else:
			os.mkdir(newdir)
	
	
	def save_thumbinal(size, image, filename):
		tmp = image.copy()
		tmp.thumbnail(size, Image.ANTIALIAS)
		tmp.save(filename, "JPEG", quality = 90)
	
	def save_crop(size, image, filename):
		img_size = image.size
		if (img_size[0] / float(size[0])) >= (img_size[1] / float(size[1])):
			#poziome przycinanie
			scale = (img_size[1] / float(size[1]))
			offset = int((img_size[0] - scale * size[0]) * 0.5)
			box = (offset, 0, img_size[0] - offset, img_size[1])
		else:
			# pionowe przycinanie
			scale = (img_size[0] / float(size[0]))
			offset =  int((img_size[1] - scale * size[1]) * 0.5)
			box = (0, offset, img_size[0], img_size[1] - offset)
			#box = (0, offset, size[0], offset + size[1]])
			pass
		tmp = image.crop(box)
		tmp.resize(size, Image.ANTIALIAS).save(filename, "JPEG", quality = 90)
	
	path = settings.SEMIRAMIS_PHOTO_DIR
	if path[-1] != '/':
		path = path + '/'
	new_photo = None
	try:
		new_photo = MarkPhoto(mark=mark, user=user, name=text)
		new_photo.save()
		mkdir(path + '%.4d' % (new_photo.id / 100))
		file_name = path + '%.4d/img%.6d' % (new_photo.id / 100, new_photo.id)
		img = Image.open(StringIO.StringIO(data))
		save_crop     (settings.SEMIRAMIS_PHOTO_THUMB, img, file_name + '_s.jpg')
		save_thumbinal(settings.SEMIRAMIS_PHOTO_THUMB, img, file_name + '_t.jpg')
		save_thumbinal(settings.SEMIRAMIS_PHOTO_LARGE, img, file_name + '_l.jpg')
		if img.format == "JPEG":
			open(file_name + '_o.jpg', 'wb+').write(data)
		else:
			img.save(file_name + '_o.jpg', "JPEG", quality = 90)
		return new_photo.id
	except:
		try:
			new_photo.delete()
		except:
			pass
		return None


##########################################################################################################################
##	Przesyla pobrany plik do Hammurabiego, zapisuje wynik w bazie.
#	
#	Funkcja przesyla plik do Hammurabiego, poprzednio jednak sprawdza czy nie 
#	jest to archiwum .zip, jesli tak jest to wysyla pokolei pliki z tego 
#	archiwum. 
#
#	Obiekty ktore dodaje do bazy dostaja identyczne opisy (pole text) i niemal
#	identyczne nazwy - rozniace sie tylko dodanym na koncu numerem lub 
#	w przypadku znacznikow opisem.
#
#	\param user uzytkownik ktory dodaje dane z pliku	
#	\param name podstawa nazwy nowododanych elemetow
#	\param text opis nowododanych elemetow
#	\param data binarny bufor zawierajacy zawartosc pliku
def save_upload(user, name, text, data):
	tracks = []
	marks = []
	is_zip = False
	link = None
	
	def save_plain(data, tracks, marks):
		link = HammurabiLink()
		tmp =  link.upload(data)
		tracks.extend(tmp[0])
		marks.extend(tmp[1])
		return tmp
		
	
	try:
		arch = zipfile.ZipFile(StringIO(data))
		is_zip = not arch.testzip() 
	except:
		pass
	
	if is_zip:
		for filename in arch.namelist():
			save_plain(arch.read(filename), tracks, marks)
	else:
		save_plain(data, tracks, marks)
		
	result_tracks = []
	result_marks = []
	
	for index, mark in enumerate(marks):
		if len(mark[3]) > 0:
			name_tmp = '%s (%d) "%s"' % (name, index + 1, mark[3])
		else:
			name_tmp = '%s (%d)' % (name, index + 1)
		new_mark = Mark(user=user, 
				name=name_tmp, 
				kind=mark[2], 
				text=text,
				title=mark[3], 
				lat=mark[0], 
				lng=mark[1]
			)
		new_mark.save()
		result_marks.append((new_mark.id, name_tmp))
	for index, track in enumerate(tracks):
		name_tmp = "%s (%d)" % (name, index + 1)
		new_track = Track(user=user,
				name=name_tmp,
				text=text,
				key=track
			)
		new_track.save()
		result_tracks.append((new_track.id, name_tmp))

	return (result_tracks, result_marks)



##	Klasa ulatwiajaca stronnicowanie danych.
#	
#	Klasa umozliwia proste, wrecz jednolinijkową obslugię
#	stronnicowania danych. 
#	Korzysta z tegu "{% pager %}" w systemie szablonow.
class Pager:
	##	Konstruktor obiektu
	#	
	#	Tworzy obiekt stronnicujacy o zadanych parametrach.
	#	Dzieki specyfice ORM (Mapowanie obiektowo-relacyjne) Django, 
	#	konkretniej leniwej ewaluacji konstruktor wykonuje sie szybko,
	#	gdyz nie pobiera z bazy calej strony (a tym bardziej tabeli).
	#	Jedyne co pobiera z bazy danych to ilosc elemetow.
	#
	#	Jesli nie istnieje strona o podanym numerze to zgłaszany jest wyjatek
	#	"HTTP 404 Page not found", nie dzieje sie tak gdy w modelu brak 
	#	jest elemetow.
	#
	#	\param model obiekt klasy podchodnej do klasy Model (czyli 
	#		zawiera dane o tabeli w bazie danych), Dane z niego beda
	#		stronnicowane.
	#	\param per_page ilosc elemetow na pojedynczej stronie, 
	#		jesli rowna 0 to ta wartosc jest pobierana z pliku 
	#		settings.py, z pola	SEMIRAMIS_ITEMS_PER_PAGE
	#	\param page numer aktualnej strony, ktora ma byc wyswietlona, 
	#		numerowanie zaczyna sie od jedynki,
	#		numer strony moze byc napisem reprezentujacym liczbe naturalna
	#	\param url napis zawierajacy szablon odnosnika do strony o podanym 
	#		numerze. powinien byc tak skonstruowany aby po uzyciu na nim 
	#		operatora formatowania ('%') w stylu printf powinno sie 
	#		otrzymywac poprawny adres strony np "/page%d/"
	def __init__(self, model, per_page, page, url):
		try:
			self.model = model
			self.per_page = int(per_page)
			if self.per_page < 1:
				self.per_page = settings.SEMIRAMIS_ITEMS_PER_PAGE
			self.page = int(page) - 1
			self.url = url
			self.hits = self.model.count()
			self.pages = (self.hits + self.per_page - 1) / self.per_page
			if self.page >= self.pages and not self.page == 0:
				raise Http404 
		except:
			raise Http404
			
			
	##	Zwraca obiekt slownika, ktory zawiera wszystkie dane potrzebne 
	#	silnikowi szablonow (a konkretniej szablonowi "{%pager%}") na
	#	wyswietlenie odnosnikow do sasiednich stron.
	#	
	#	\param adjacent_pages ilosc sasiadujacych stron, do ktorych maja byc 
	#		wyswietlane odnosniki, np jesli jest rowne 5 to oprocz bierzacej
	#		strony sa wyswietlane 2 poprzedzajace i 2 nastepujace po 
	#		(oczywiscie jesli istnieja).
	#		Jesli wartosc nie jest podana to pobierane sa ustawienia z pliku 
	#		settings.py, z pola	SEMIRAMIS_ADJACENT_PAGES
	def pager(self, adjacent_pages = None):
		if adjacent_pages == None:
			adjacent_pages = settings.SEMIRAMIS_ADJACENT_PAGES
		first = prev = next = last = None
		page = self.page
		pages = self.pages
		if page > (adjacent_pages / 2):
			first = {'i' : 1, 'url' : self.url % 1}
		if page > 0:
			prev = {'i' : page, 'url':self.url % page}
		if pages - page > 1:
			next = {'i' : page + 2, 'url' : self.url % (page + 2)}
		if pages - page -  adjacent_pages / 2 > 1:
			last = {'i' : pages, 'url':self.url % pages}
		#r = range(max(0, self.page - adjacent_pages), min(self.pages, self.page + adjacent_pages + 1))
		start = max(0, self.page - adjacent_pages / 2)
		stop = min(self.pages, start + adjacent_pages )
		l = [{'i':(i + 1), 'url':self.url % (i+1)} for i in xrange(start, stop)]
		return {'page': self.page + 1,
				'pages': self.pages,
				'per_page': self.per_page,
				'hits': self.hits,
				'page_numbers': l,
				'first': first,
				'prev': prev,
				'next': next,
				'last': last
			}
			
			
	##	Pobiera z bazy dane ktore znajduja sie na bierzacej stronie.
	# 
	#	Dopiero w tej metodzie wykonywane jest wlasciwe zapytanie do bazy danych
	#	oraz pobierane sa dane.
	def data(self):
		f, t = self.page * self.per_page, (self.page + 1) * self.per_page
		result= self.model[f:t]
		return result

