# -*- coding: utf-8 -*-

##
#	Plik zawierajacy widoki Django czyli w praktyce najwazniejszy plik 
#	w Semiramidzie. 
#
#	\author Krzysztof Kosyl

from django.http import HttpResponse
from django.http import HttpResponseRedirect, HttpResponseForbidden
from django.http import Http404
from django.conf import settings 
from django.db.models.manager import Manager
from django import oldforms as forms
from django import newforms
from django.utils.translation import gettext_lazy
#from django.newforms import form_for_model, form_for_instance, save_instance, BaseForm, Form, CharField

from django.shortcuts import render_to_response, get_object_or_404
from django.contrib.auth.decorators import login_required

from django.template import Context, loader

from django.contrib.auth.models import User

from django.contrib.auth.forms import UserCreationForm  #??
from django.core.paginator import ObjectPaginator, InvalidPage



from datetime import datetime
import os
from PIL import Image
try:
	import cStringIO as StringIO
except ImportError:
	import StringIO

from nebuchadrezzar.semiramis.models import Track, TrackComment, Mark, MarkComment, MarkPhoto
from nebuchadrezzar.semiramis.templatetags.semiramis_templatetags import link_to, babel

from hammurabi_link import HammurabiLink
from utils import post_data, common, delete_object_or_forbidden, edit_object_or_forbidden, save_photo, save_upload, Pager
import socket





###############################################################################
# Klasy formularzy
###############################################################################

mark_kinds=[
	(20, gettext_lazy('Other')),
	(21, gettext_lazy('Shop')),
	(22, gettext_lazy('Camp site')),
	(23, gettext_lazy('Sanitary object')),
	(24, gettext_lazy('Beach, lido')),
	(25, gettext_lazy('Sacral object'))
]



class MarkForm(newforms.Form):
	lat = newforms.CharField(label=gettext_lazy("Latitude"), required=True)
	lng = newforms.CharField(label=gettext_lazy("Longitude"), required=True)
	kind = newforms.ChoiceField(label=gettext_lazy("Kind"), choices=mark_kinds)
	name = newforms.CharField(label=gettext_lazy("Name"), required=True)
	text = newforms.CharField(label=gettext_lazy("Description"), widget=newforms.Textarea(attrs={'rows': 10, 'cols': 40}), required=False)

class TrackForm(newforms.Form):
	name = newforms.CharField(label=gettext_lazy("Name"), required=True)
	color = newforms.CharField(label=gettext_lazy("Color"), required=False, max_length=3)
	text = newforms.CharField(label=gettext_lazy("Description"), widget=newforms.Textarea(attrs={'rows': 10, 'cols': 40}), required=False)
	

class CommentForm(newforms.Form):
	text = newforms.CharField(label=gettext_lazy("Comment"), widget=newforms.Textarea(attrs={'rows': 10, 'cols': 40}), required=True)
	
class PhotoForm(newforms.Form):
	name = newforms.CharField(label=gettext_lazy("Name"), required=True)

class PhotoUploadForm(newforms.Form):
	text = newforms.CharField(label=gettext_lazy("Description"), required=True)
	photo_data = newforms.Field(label=gettext_lazy("File"), widget=newforms.FileInput())
	
class DataUploadForm(newforms.Form):
	name = newforms.CharField(label=gettext_lazy("Name"), required=True)
	text = newforms.CharField(label=gettext_lazy("Description"), widget=newforms.Textarea(attrs={'rows': 10, 'cols': 40}), required=False)
	file_data = newforms.Field(label=gettext_lazy("File"), widget=newforms.FileInput())


###############################################################################
# funkcje widoku
###############################################################################


def http500(request):
	return render_to_response('http404.html', common(request, {}))

def http404(request):
	return render_to_response('http500.html', common(request, {}))

##	
#	Widok strony glownej.
#	Wyswietla najnowsze z dodanych obiektow.
def main(request):
	tracks = Track.objects.order_by('-date')[:10]
	marks = Mark.objects.order_by('-date')[:10]
	photos = MarkPhoto.objects.order_by('-date')[:10]
	users = User.objects.order_by('-date_joined')[:10]

	return render_to_response('main.html', common(request, {
			'tracks': tracks, 
			'marks':marks, 
			'photos':photos,
			
			'main': True,
			'users':users
	}))


##	
#	Widok mapy.
#	sprawdza czy przegladarka to IE, jesli tak to umieszcza na stronie wymagane
#	przez ta przegladarkie polecenia.
def gmap(request):
	isie = "MSIE" in request.META['HTTP_USER_AGENT']
	gk = settings.SEMIRAMIS_GOOGLE_KEY
	google_key = [{'host':i, 'key':gk[i]} for i in gk]
	
	return render_to_response('map.html', common(request, {
			'isie':isie,
			'google_key':google_key
		}))

##	
#	Widok listy sciezek.
#	Wyswietla liste sciezek w kolejnosci poczawszy od najnowszych.
def track_page(request, page = "1"):
	pager = Pager(Track.objects.all(), 0, page, '/track/page%d/')
	return render_to_response('new/browse.html', common(request, {
			'items':pager.data(), 
			'pager': pager.pager(),
			'view': 'track'
		}))


##	
#	Widok pojedynczej sciezki.
#	Wyswietla informacje o scierzce, komentarze oraz jesli uzytkownik jest 
#	zalogowany - formularz dodawania komentarzy.
def track_id(request, id):
	try:
		track_data = Track.objects.get( id = int(id) )
		track_comment = track_data.trackcomment_set.all()
	except :
		raise Http404
	
	comment_form = None
	
	if request.user.is_authenticated():
		if request.method == 'POST':
			comment_form = CommentForm(request.POST)
			if comment_form.is_valid():
				new_comment = TrackComment(
						track=track_data, 
						user=request.user,
						text=comment_form.clean_data['text'], 
				)
				new_comment.save()
					
			return HttpResponseRedirect(".")	
		else:
			comment_form = CommentForm()

	return render_to_response('new/item.html', common(request, {
			'item':track_data,
			'comments':track_comment,
			'comment_form':comment_form,
			'view': 'track'
		}))


@login_required
def track_id_edit(request, id):
	return edit_object_or_forbidden(Track, TrackForm, None, request, id=int(id))

	
	
@login_required
def track_id_delete(request, id):
	return delete_object_or_forbidden(Track, request, id=int(id))


@login_required
def track_comment_id_edit(request, id):
	return edit_object_or_forbidden(TrackComment, CommentForm, None, request, id=int(id))


@login_required
def track_comment_id_delete(request, id):
	return delete_object_or_forbidden(TrackComment, request, id=int(id))


@login_required
def mark_add(request):
	mark_form = None
	if request.method == 'POST':
		mark_form = MarkForm(request.POST)
		if mark_form.is_valid():
			clean = mark_form.clean_data
			new_mark = Mark(user=request.user, name=clean['name'], text=clean['text'],
					lat=clean['lat'], lng=clean['lng'], kind=clean['kind'])
			new_mark.save()
		return HttpResponseRedirect(link_to(new_mark))

	else:
		try:
			lat = "%.6f" % float(request.GET['lat'])
			lng = "%.6f" % float(request.GET['lng'])
			
			mark_form = MarkForm({'lat':lat, 'lng':lng, 'name':'Nowy punkt', 
				'text':'Opis punktu', 'kind':20})
		except:
			mark_form = MarkForm()

	return render_to_response('mark_add.html', common(request, {
			'mark_form':mark_form
		}))


def mark_page(request, page = "1"):
	pager = Pager(Mark.objects.all(), 0, page, '/mark/page%d')
	return render_to_response('new/browse.html', common(request, {
			'items':pager.data(), 
			'pager': pager.pager(),
			'view': 'mark'
		}))
	

def mark_id(request, id):
	try:
		mark_data = Mark.objects.get(id = int(id))
		mark_comment = mark_data.markcomment_set.all()
		mark_photo = mark_data.markphoto_set.all()
	except :
		raise Http404
	
	comment_form = None
	photo_form = None
	
	
	if request.user.is_authenticated():  # zmienić na have_permision (upload)?
		if request.method == 'POST':
			kind =  request.POST['kind']
			if kind == 'comment':
				comment_form = CommentForm(request.POST)
				if comment_form.is_valid():
					new_comment = MarkComment(id=None, mark=mark_data, user=request.user,
							text=comment_form.clean_data['text'], date=datetime.now())
					new_comment.save()
					
			elif kind == 'photo':
				photo_form = PhotoUploadForm(post_data(request))
				if photo_form.is_valid():
					clean = photo_form.clean_data
					result = save_photo(mark_data, request.user, clean['text'], clean['photo_data']['content'])
					
			return HttpResponseRedirect(".")	

		else:
			comment_form = CommentForm()
			photo_form = PhotoUploadForm()

	return render_to_response('new/item.html', common(request, {
			'item':mark_data,
			'comments': mark_comment, 
			'photos': mark_photo, 
			'comment_form':comment_form, 
			'photo_form':photo_form,
			'view': 'mark'
		}))

@login_required
def mark_id_edit(request, id):
	return edit_object_or_forbidden(Mark, MarkForm, None, request, id=int(id))


@login_required
def mark_id_delete(request, id):
	return delete_object_or_forbidden(Mark, request, id=int(id))


@login_required
def mark_comment_id_edit(request, id):
	return edit_object_or_forbidden(MarkComment, CommentForm, None, request, id=int(id))


@login_required
def mark_comment_id_delete(request, id):
	return delete_object_or_forbidden(MarkComment, request, id=int(id))
	
	
def user_login(request, login):
	tracks = Track.objects.filter(user__username=login).order_by('-date')[:10]
	marks = Mark.objects.filter(user__username=login).order_by('-date')[:10]
	photos = MarkPhoto.objects.filter(user__username=login).order_by('-date')[:10]
	
	return render_to_response('main.html', common(request, {
			'user':request.user,
			'tracks': tracks, 
			'marks':marks, 
			'photos':photos,
	}))






@login_required
def upload(request):
	upload_form = None
	if request.method == 'POST':
		upload_form = DataUploadForm(post_data(request))
		if upload_form.is_valid():
			clean = upload_form.clean_data
			tmp = save_upload(request.user, 
					clean['name'], 
					clean['text'], 
					clean['file_data']['content']
				)
			request.session['upload_result'] = tmp
			
		return HttpResponseRedirect("/upload/result/")	
	else:
		upload_form =  DataUploadForm()

	return render_to_response('upload/upload.html', common(request, {
			'upload_form':upload_form
		}))


@login_required
def upload_result(request):
	tracks = None
	marks = None
	if 'upload_result' in request.session:
		tracks, marks = request.session['upload_result']
		del request.session['upload_result']
	return render_to_response('upload/upload_result.html', common(request, {
			'tracks':tracks, 
			'marks':marks
		}))


def photo_id(request, id):
	photo = MarkPhoto.objects.get(id = int(id))
	return render_to_response('photo_id.html', common(request, {
			'photo':photo
		}))

@login_required
def photo_id_edit(request, id):
	return edit_object_or_forbidden(MarkPhoto, PhotoForm, None, request, id=int(id))


@login_required
def photo_id_delete(request, id):
	return delete_object_or_forbidden(MarkPhoto, request, id=int(id))


@login_required
def accounts_profile(request):
	return HttpResponseRedirect(link_to('/'))
	#return render_to_response('accounts/profile.html', {'user':request.user})

##
#	
def accounts_register(request):
	form = UserCreationForm()
	
	if request.method == 'POST':
		data = request.POST.copy()
		errors = form.get_validation_errors(data)
		if not errors:
			new_user = form.save(data)
			return HttpResponseRedirect("/accounts/created/")
	else:
		data, errors = {}, {}
	
	return render_to_response("accounts/register.html", common(request, {
			'form' : forms.FormWrapper(form, data, errors)
		})) 


def accounts_created(request):
	return HttpResponseRedirect("/accounts/login/")



##	
#	Widok wytwarzajacy dane o wszystkich sciezkach z danego obaszaru
def data_tracks(request):
	from GMapEncode import encodePolyline
	
	latA = float(request.GET['latA'])
	lngA = float(request.GET['lngA'])
	latB = float(request.GET['latB'])
	lngB = float(request.GET['lngB'])
	zoom = int(request.GET['zoom'])
	
	result = []
		
	keys = {}
	link = HammurabiLink()
	tmp =  link.query(latA=latA, lngA=lngA, latB=latB, lngB=lngB)
	
	for i in tmp:
		if keys.has_key(i[0]):
			keys[i[0]].append((i[1], i[2]))
		else:
			keys[i[0]] = [(i[1], i[2])]
	
	tracks = Track.objects.filter(key__in=keys.keys()).order_by('name')
	
	for track in tracks:
		color = '#ff0000'
		if len(track.color) == 3:
			t = track.color
			color = '#' + t[0] * 2 + t[1] * 2 + t[2] * 2
		name = track.name.replace(' ', r'\x20')
		buf = ['%d %s "%s"' % (track.id, color, name)]
		for part in keys[track.key]:
			encoded = encodePolyline(part[1])
			buf.append('%d %s %s' % (part[0], encoded[0], encoded[1]))
		result.append('\n'.join(buf))
	result = '\n#-#-#\n'.join(result)
	return HttpResponse(result, mimetype="text/plain")
		
		
##	
#	Widok wytwarzajacy dane o wszystkich markach z danego obszaru
def data_marks(request):
	result = ''
	try:
		latA = float(request.GET['latA'])
		lngA = float(request.GET['lngA'])
		latB = float(request.GET['latB'])
		lngB = float(request.GET['lngB'])
		zoom = int(request.GET['zoom'])
		sev = 2
		if zoom >= 8:
			sev = 1
		if zoom >= 12:
			sev = 0
		marks = Mark.objects.filter(
				lat__gt=latA, 
				lng__gt=lngA, 
				lat__lt=latB, 
				lng__lt=lngB, 
				severity__gte=sev
			).order_by('name')
		result = '\n'.join(["%d:%.6f:%.6f:%d:%d:%s" % (mark.id, mark.lat, mark.lng, mark.kind, mark.severity, mark.name) for mark in marks])
	except:
		pass

	return HttpResponse(result, mimetype="text/plain")


##	
#	Widok wytwarzajacy dane dane opisy marka w wyskakujacym baloniku
def data_mark(request):
	#try:
		item = Mark.objects.get(id=int(request.GET['id']))
		photos = item.markphoto_set.all()[:2]
		return render_to_response('data_mark.html', {'item': item, 'photos': photos})
	#except:
		return HttpResponse('#', mimetype="text/html")
		
		
##	
#	Widok eksportujacy dane do podanego formatu.
#
#	\param data jesli zaczyna sie przecinkiem to po nim wystepuje numer 
#		scierzki ktora ma byc wyeksportowana, w przeciwnym wypadku 
#		zwiera liste liczb oddzielonych przecinkami z ktrorych 5 pierwszych
#		okresla obszar a pozostale numery sciezek krore maja byc wyeksportowane  
#	\param format format pliku 
def data_export(request, data, format):
	mime = {
			'kml':'application/vnd.google-earth.kml+xml',
			'pdf':'application/pdf',
			'ps' :'application/postscript',
			'gpx':'application/gpx+xml',
			'txt':'application/octet-stream',
		}
	link = HammurabiLink()
	if data[0] == ',':
		track = Track.objects.get(id=int(data[1:]))
		query = 'track\nformat:%s\nid:%s\n' % (format, track.key)
	else:
		data = data.split(',')
		query = 'latA:%.6f\nlngA:%.6f\nlatB:%.6f\nlngB:%.6f\n' % (
				float(data[0]),
				float(data[1]),
				float(data[2]),
				float(data[3])
			)
		ids = map(int, data[5:])
		if len(ids) > 0:
			tracks = Track.objects.filter(id__in=ids)
			keys = [str(t.key) for t in tracks]
		else:
			keys = ['-1']
		keys = ' '.join(keys)
		query = 'track\nformat:%s\n%sid:%s\n' % (format, query, keys)
	r = link.socket_get(query)
	return HttpResponse(r, mimetype=mime[format])

##
#	Generuje podglad tekstu
def data_babel(request):
	try:
		print request.POST
		text = babel(request.POST['text'])
		return HttpResponse(text, mimetype="text/html")
	except:
		return HttpResponse(' ', mimetype="text/html")

@login_required
def sandbox(request, path):
	raise Http404

