# -*- coding: utf-8 -*-

##
#	Plik zawierajacy obslugie XML-RPC.
#
#	\author Krzysztof Kosyl


from SimpleXMLRPCServer import SimpleXMLRPCDispatcher
from django.http import HttpResponse
from django.shortcuts import render_to_response

from nebuchadrezzar.semiramis.models import Track, TrackComment, Mark, MarkComment, MarkPhoto
from hammurabi_link import HammurabiLink

dispatcher = None
try:
	#python 2.5
	dispatcher = SimpleXMLRPCDispatcher(allow_none=False, encoding=None)
except TypeError:
	#python 2.4
	dispatcher = SimpleXMLRPCDispatcher()


def getTracks(first,last):
	"""
getTracks(first,last)

Zwraca sciezki w postaci listy slownikow,
elementy o numerach [first, last)
	"""
	
	result = []
	for i in Track.objects.all().order_by('id')[first:last]:
		result.append({
				'id': i.id,
				'name': i.name
			})
	return result
	
	
def getMarks(first, last):
	"""
getMarks(first, last) 

Zwraca znaczniki w postaci listy slownikow,
elementy o numerach [first, last)
	"""
	
	result = []
	for i in Mark.objects.all().order_by('id')[first:last]:
		result.append({
				'id': i.id,
				'name': i.name,
				'lat': i.lat,
				'lng': i.lng
			})
	return result
	
def getTrackPoints(id):
	"""
getTrackPoints(id)

Zwraca wsporzedne punktow sciezki o podanym
id jako liste. W tej liście po kolei 
znajduja sie liczby okreslajace 
szerokosc i dlugosc geograficzna.
	"""


	result = []
	track = Track.objects.get(id = id)

	link = HammurabiLink()
	data = link.socket_get('track\nformat:txt\nid:%d\n' % track.key)
	data = data.split('\n')
	for line in data:
		try:
			s = line.split(':')
			result.append(int(s[0]) / 1000000.0)
			result.append(int(s[1]) / 1000000.0)
		except:
			pass
	return result	

dispatcher.register_function(getTracks, 'getTracks')
dispatcher.register_function(getMarks, 'getMarks')
dispatcher.register_function(getTrackPoints, 'getTrackPoints')

def xmlrpc(request):
	response = HttpResponse()
	if len(request.POST):
			response.write(dispatcher._marshaled_dispatch(request.raw_post_data))
			response['Content-length'] = str(len(response.content))
			return response
	else:
			result = []
			methods = dispatcher.system_listMethods()
			for method in methods:
					result.append({
							'name': method,
							'help': dispatcher.system_methodHelp(method)
						})
						
			return render_to_response('xmlrpc.html', {'methods': result})
