/**
	Kod kliencki Semiramidy.
		
	Działa w przegladarce uzytkownika podczas wyswietlania mapy. 
	Zajmuje sie ladowaniem danych z Semiramidy (korzystajac przy tym
	z technologii AJAX). Ponadto obsługuje efekty animacji i tooltipow 
	w przegladarce.
	
	Jest zalezny od biblotek jQuery i jQuery.Accordion wiec powinien byc
	zadowany po nich.
	
	\author Krzysztof Kosyl
*/
	
	/**
		Zmienne globalne, używane niejawnie przez funkcje.
		Aby zapobiec pomylka ich nazwy zaczynaja sie na "glob"
	*/
	
	var globMap;
	var globTracks = new Array();
	var globMarks = new Array();
	var globIcons = new Array();
	var globMarkerVisible = new Array();
	var globMode = 'kind';
	var globAngleFormat = 'dms';
	var globSelectedTrack = -1;
	var globLastMouse;
	var globZoom;
	var globSpeed = 'normal';
	var globMinimalized = false;
	var globUpdateTimer;
	
	/*
		Formatuje liczbe zmiennoprzecinkową.
		
		\param number wartosc do sformatowania
		\param size ilosc cyfr przed przecinkiem
		\param prec ilosc cyfr po kropce (jesli 0 to kropka jest pomijana)
	*/
	function formatNumber(number, size, prec) {
		var used = 1;
		if (number >= 10.0)
			used = 2;
		if (number >= 100.0)
			used = 3;
		var prefix = '';
		for (var i = 0; i < (size - used); i++)
			prefix += '0';
		return prefix + number.toFixed(prec);
		
	}
	
	/**
		Formatuje 
	
	*/	
	function formatAngle(a, c) {
		var sign;
		if (a < 0) {
			a = -a;
			sign = c.charAt(0);
		}
		else {
			sign = c.charAt(1);
		}
		switch (globAngleFormat) {
			case 'dms':
				h = Math.floor(a);
				m = Math.floor((a - h) * 60);
				s = (a * 60 - m - h * 60) * 60;
				return formatNumber(h,3,0) + '\xb0' + formatNumber(m, 2, 0) + "\'" + formatNumber(s, 2, 2) + "\"" + sign;
			case 'dm':
				h = Math.floor(a);
				m = (a - h) * 60;
				return formatNumber(h,3,0) + '\xb0' + formatNumber(m, 2, 4) + "\'" + sign;
			case 'd':
			default:
				return formatNumber(a,3,5) + '\xb0' + sign; 
		}
	}
	
	function formatLatLng(lat, lng, sep) {
		if (typeof(sep) != 'number') {
			lng = lat.lng();
			lat = lat.lat(); 
		}
		sep = typeof(sep) != 'undefined' ? sep : " ";
		return formatAngle(lat, 'SN') + sep + formatAngle(lng, 'WE');
	}
	

	
	function mouseHandler(latlng) {
		if (typeof(latlng) != 'undefined')
			globLastMouse = latlng;
		else
			latlng = globLastMouse;
		$("#status_mouse").text(formatLatLng(latlng));
	}
	
	/**
	 * Zapisuje cookie określające aktualne polożenie.
	 * Funkcja wywołuje się zarówno po zdarzeniu "moveend" jak i "zoomend", 
	 * wynika to z specyfiki GoogleMapsApi
	 * Ustawia dane w zakladce 'Pozycja'
	 */
	function moveendHandler() {
		var center = globMap.getCenter();
		var val = center.lat() + ":" + center.lng() + ":" + globMap.getZoom();
		$.cookie('center', val, {expires: 365, path: '/'});
		
		var center = globMap.getCenter();
		var bounds = globMap.getBounds();
		var southWest = bounds.getSouthWest();
		var northEast = bounds.getNorthEast();

		$("#status_center").text(formatLatLng(center));
		$("#status_boundsA").text(formatLatLng(southWest));
		$("#status_boundsB").text(formatLatLng(northEast));
		
		$("#mark_add").attr("href","/mark/add/?lat=" + center.lat().toFixed(6) + "&lng=" + center.lng().toFixed(6));
		
		updateOverlays();
	}
	
	function setAngleFormat(arg) {
		globAngleFormat = arg;
		moveendHandler();
		mouseHandler();
		$.cookie('angleFormat', arg, {expires: 365, path: '/'});
		return false;
	}
	
	
	function initMap() {
		if (GBrowserIsCompatible()) {
			globMap = new GMap2(document.getElementById("map"),{logoPassive:true});
			
			globMap.addControl(new GMapTypeControl());
			globMap.addControl(new GLargeMapControl(), new GControlPosition(G_ANCHOR_TOP_RIGHT,new GSize(40,70)));
			globMap.addControl(new GScaleControl(), new GControlPosition(G_ANCHOR_BOTTOM_RIGHT,new GSize(20,30)));
			
			var cookie = $.cookie('center');
			if (cookie != null) {
				d = cookie.split(":")
				var lat = d[0] * 1.0;
				var lng = d[1] * 1.0;
				var z =  d[2] * 1;
				if (
					(lat > -90) && (lat < 90) &&
					(lng > -180) && (lng < 180) &&
					(z >= 0) && (z < 30)
					)
					globMap.setCenter(new GLatLng(d[0] * 1.0, d[1] * 1.0), d[2] * 1);
				else
					globMap.setCenter(new GLatLng(53.009839, 18.594468), 11);
			}
			else {
				globMap.setCenter(new GLatLng(53.009839, 18.594468), 11);
			}

			GEvent.addListener(globMap, "mousemove", mouseHandler);
			
			GEvent.addListener(globMap, "moveend", moveendHandler);
			moveendHandler();
			
			$("img").each(function(i){
				var src = $(this).attr("src");
				var re = new RegExp("http://www.google.com/intl/[^/]*/mapfiles/poweredby.png","i");
				if (re.test(src)) {
					$(this).css({position:'absolute',  right:'150px', bottom:'23px', left:''});
				}
			});
		}
	}
	/**
		Funkcja zwracajaca slownik z aktualną pozycja na mapie.
		Uzywana jest przez funkcje tracksDownload() i marksDownload()
	*/	
	function getBounds() {
		var X = globMap.getBounds();
		var A = X.getSouthWest();
		var B = X.getNorthEast();
		
		return { 
			rand: 'f' + Math.floor(Math.random() * 99999),
			latA: A.lat().toFixed(6),
			lngA: A.lng().toFixed(6),
			latB: B.lat().toFixed(6),
			lngB: B.lng().toFixed(6),
			zoom: globMap.getZoom()
   		};
	}
	

	
	function getColor(color, kind) {
		switch (globMode) {
			case 'id':
				return color;
			case 'random':
				h = Math.floor(Math.random() * 0x1000000).toString(16);
				return ('#' + h + '000000').slice(0,7);
			case 'kind':
				switch (kind) {
					case  0: return "#5ab4eb";
					case  1: return "#8000c0";
					case  2: return "#0000c0";
					case  3: return "#006000";
					case  4: return "#00c060";
					case  5: return "#ffff00";
					case  6: return "#ff8000";
					case  7: return "#c00000";
					default: return "#808080";
				}
			case 'one':
				return "#ff0000";
		default:
				return "#ff0000";
		}
	}
	
	/**
		Pobiera z serwera dane o sciezkach i wyswietla je.
		
		Za pomoca AJAX pobiera z serwera dane o ścierzkach, spelniajacych warunki:
		- znajduja sie na obszarze aktualnie wyswietlanym
		- nie są już sciągniete i wyswietlone
		Nastepnie wyswietla je uwzgledniajac aktualnie wybrany tryb wyswietlania.
	*/
	function tracksDownload() {
		var downloaded = new Array();
		var request = getBounds();
		var omit = '';
		for (var id in globTracks)
			omit += id + ',';
		request.omit = omit;
		$.get("/data/tracks/",
			request,
			function(data){

				if (data.indexOf('\"') >= 0) {
					var tracks = data.split("\n#-#-#\n");
					for (t in tracks) {
						var track = tracks[t]; 
						var lines = track.split("\n");
						var desc = lines[0].split(' ');
						var id = desc[0] * 1;
						var color = desc[1];
						var name = eval("(" + desc[2] + ")");
						
						downloaded[id] = true; 
						
						if (typeof(globTracks[id]) != 'undefined')
							continue;
						
						globTracks[id] = {
								id: id, 
								color: color, 
								name: name, 
								parts: [], 
								visible: true
							}
						for (var i = 1; i < lines.length; i++) {
							var line = lines[i].split(' ');
							var kind = line[0] * 1;
							var part = {
								color: getColor(color, kind),
								weight: 2,
								opacity: 0.7,
								points: line[1],
								levels: line[2],
								zoomFactor: 2,
								numLevels: 18,
								kind: kind,
								orgcolor: color
							};
							var polyline = new GPolyline.fromEncoded(part);
							part.overlay = polyline;
							globTracks[id].parts.push(part);
							globMap.addOverlay(polyline);
						}
					}
				}
				//usuniecie zbednych danych
				tmpTracks = new Array();
				for (t in globTracks) {
					var track = globTracks[t];
					if (typeof downloaded[track.id] != 'undefined') {
						tmpTracks[t] = track;	
					}
					else {
						var parts = globTracks[t].parts;
						for (p in parts) {
							var part = parts[p];
							globMap.removeOverlay(part.overlay);
						}
					}
				}
				globTracks = tmpTracks;
		
				// wyczyczenie listy i aktualizacja
				$("#tracks_listing").empty();
				for (t in globTracks) {
					var track = globTracks[t];
					var id = track.id;
					$("#tracks_listing").append(
							"<input type='checkbox' name='track_select' " + 
 							( track.visible ? " checked='checked' " : "style='color:red'") +
							" onclick='return trackVisible(" + id + ")' />" + 
							"<a href='/track/" + id + "/'>" + track.name + "</a>" +
							"<a href='.'  onclick='" + 
							"return trackSelect(" + id +")'> " + 
							"<img src='/static/icons/goto.png'></a><br/>\n"
						);
				}
			}
		);
	}
	
	/**
		Zmienia tryb wyswietlania scierzek.
		
		\param mode wybrany tryb
	*/
	function tracksMode(mode) {
		globMode = mode;
		var random = (globMode == 'random')
		for (t in globTracks) {
			var parts = globTracks[t].parts;
			var c = getColor(0, 0);
			for (p in parts) {
				var part = parts[p];
				globMap.removeOverlay(part.overlay);
				part.color = random ? c : getColor(part.orgcolor, part.kind);
				part.overlay = new GPolyline.fromEncoded(part);
				if (globTracks[t].visible)
					globMap.addOverlay(part.overlay);
			}
		}
		return false;
	}
	
	function trackInfo(id) {
		var len = 0;
		var parts = globTracks[id].parts;
		for (p in parts) {
			var part = parts[p];
			for (var i = 1; i < part.overlay.getVertexCount(); i++) {
				var from = part.overlay.getVertex(i - 1);
				var to = part.overlay.getVertex(i);
				len += to.distanceFrom(from);
			}
		}
		$("#tracks_selected").html("<b>" + formatNumber(len / 1000.0, 1, 3) + "km </b></br>" + globTracks[id].name);
	}
	
	function trackWeightOpacity(id, weight, opacity) {
		track = globTracks[id];
		var parts = track.parts;
		for (p in parts) {
			var part = parts[p];
			
			globMap.removeOverlay(part.overlay);
			part.weight = weight;
			part.opacity = opacity;
			part.overlay = new GPolyline.fromEncoded(part);
			globMap.addOverlay(part.overlay);
		}
		return false;
	}
	

	
	/**
		Wyróżnia na mapie ścierzkię o podanym identyfikatorze.
		
		Zwieksza grubosć lini ścierzki, zmienia skale tak aby scierzka byla jak najlepiej widoczna,
		centruje mapę na ścieżce.
		
		\param id identyfikator scierzki
	*/
	function trackSelect(id) {
		if (globSelectedTrack >= 0)
			trackWeightOpacity(globSelectedTrack, 2, 0.7);
		globSelectedTrack = id;
		trackWeightOpacity(globSelectedTrack, 5, 1.0);
		/** centrowanie */
		var minLat, minLng, maxLat, maxLng;
		track = globTracks[id];
		var parts = track.parts;
		var bounds = new GLatLngBounds();
		for (p in parts) {
			var overlay = parts[p].overlay;
			var count = overlay.getVertexCount();
			for (var i = 0; i < count; i++)
				bounds.extend(overlay.getVertex(i));
		}
		/* troche sie zacina
		globMap.setZoom(globMap.getBoundsZoomLevel(bounds));
		globMap.panTo(bounds.getCenter());
		*/
		globMap.setCenter(bounds.getCenter(), globMap.getBoundsZoomLevel(bounds));
		trackInfo(id);
		return false;
	}
	
	
	/**
		Przywraca wyrórznioną ścierzkie do normalnego wyglądu.
		
		Przywraca ścierzkię do poprzedniego wyglądu, zmieniając jej grubość na standardową.
	*/
	function trackUnselect() {
		if (globSelectedTrack >= 0)
			trackWeightOpacity(globSelectedTrack, 2, 0.7);
	}
	
	function trackVisible(id, val) {
		track = globTracks[id];
		var show = track.visible;
		if (typeof(val) != 'undefined')
			show = !val;

		var parts = track.parts;
		if (show) {
			for (p in parts) {
				var part = parts[p];
				globMap.removeOverlay(part.overlay);
			}
		}
		else {
			for (p in parts) {
				var part = parts[p];
				globMap.addOverlay(part.overlay);
			}
		}
		track.visible = !show;
		return true;
	}
	
	function trackVisibleAll() {
		$("input[@name=track_select]").attr("checked", "checked");
		for (var id in globTracks)
			trackVisible(id, true);
		return false;
	}
	
	function trackVisibleNone() {
		$("input[@name=track_select]").removeAttr("checked");
		for (var id in globTracks)
			trackVisible(id, false);
		return false;
	}
	
	
	
	function move(lat,lng) {
		globMap.panTo(new GLatLng(lat,lng));
		return false;
	}
	
	function marksListingReload() {
		$("#marks_listing").empty();
		for (var id in globMarks) {
			if (globMarkerVisible[globMarks[id].kind]) {
				var lat = globMarks[id].lat;
				var lng = globMarks[id].lng;
				var name = globMarks[id].name;
				$("#marks_listing").append(
						"<a href='/mark/" + id + "/'>" + name + "</a>" +
						"<a href='.'  class='mark_goto' onclick='" + 
						"return move(" + lat + "," + lng + ")'>" + 
						"<img src='/static/icons/goto.png'></a><br/>\n"
					);
			}
		}
	}
	
	function createMarker(id, lat,lng, kind, name){
		var point = new GLatLng(lat, lng);
		var marker = new GMarker(point, {title:name, icon: globIcons[kind]});
		GEvent.addListener(marker, "click", function() {
				$.get("/data/mark/",
					{'id':id},
					function(data){
						//alert(data);
						if (data != '#')
						marker.openInfoWindowHtml(data);
					});
				//marker.openInfoWindowHtml("Marker <b>" + id + "</b>");
			});
		if (globMarkerVisible[kind]) {
			globMap.addOverlay(marker);
		}
		return marker;
	}
	
	function marksDownload() {
		var request = getBounds();
		$.get("/data/marks/",
			request,
			function(data){
				//alert(data);
				for (m in globMarks) {
					globMap.removeOverlay(globMarks[m].overlay);
				}
				//$("#marks_listing").empty();
				globMarks.length = 0; //clear()
				if (data.indexOf(':') < 0)
					return;
				var lines = data.split("\n");
				for (i in lines) {
					var d = lines[i].split(':');
					//alert(d);
					var id = d[0] * 1;
					var lat = d[1] * 1.0;
					var lng =d[2] * 1.0;
					var kind = d[3] * 1;
					var severity = d[4] * 1;
					var name = d[5];
					globMarks[id] = {
							id: id,
							lat: lat,
							lng: lng,
							kind: kind,
							severity: severity,
							name: name
						};
					//alert(globMarks[id]);
// 					var point = new GLatLng(lat, lng);
// 					var marker = new GMarker(point, {title:name, icon: globIcons[kind]});
// 					$("#marks_listing").append(
// 							"<a href='/mark/" + id + "/'>" + name + "</a>" +
// 							"<a href='.'  class='mark_goto' onclick='" + 
// 							"return move(" + lat + "," + lng + ")'>" + 
// 							"<img src='/static/icons/goto.png'></a><br/>\n"
// 						);
					//globMarks[id].overlay = marker;
// 					GEvent.addListener(marker, "click", function() {
// 						marker.openInfoWindowHtml("Marker <b>x</b>");
// 					});
					
// 					GEvent.addListener(marker, "click", function() {
//     					marker.openInfoWindowHtml("Marker #<b>x</b>");
//   					});
// 					if (globMarkerVisible[kind]) {
// 						globMap.addOverlay(marker);
// 					}
					globMarks[id].overlay = createMarker(id, lat,lng, kind, name);
					//globMarks[id].overlay = marker;
				}
				marksListingReload();
		});
	}
	
	function markVisibility(kind) {
		if (globMarkerVisible[kind]) {
			$("#marker" + kind).attr('src', '/static/marker/icon' + kind + 'off.png');
			for (var i in globMarks) {
				if (globMarks[i].kind == kind)
					globMap.removeOverlay(globMarks[i].overlay);
			}
				
			globMarkerVisible[kind] = false;
		}
		else {
			$("#marker" + kind).attr('src', '/static/marker/icon' + kind + 'on.png');
			for (var i in globMarks) {
				if (globMarks[i].kind == kind)
					globMap.addOverlay(globMarks[i].overlay);
			}
			globMarkerVisible[kind] = true;
		}
		marksListingReload();
	}
	
	
	/**
		Aktualizuje sciezki i znaczniki natychmiastowo. 
	*/
	function doUpdateOverlays() {
		//alert('doUpdateOverlays()');
		tracksDownload();
		marksDownload()
	}
	
	/**
		Aktualizuje sciezki i znaczniki, odczekujac chwile tak aby nie 
		powodowac niepotrzebnego obciarzenia gdy zmienianie pozycji jest 
		w trakcie. 
	*/
	function updateOverlays() {
		window.clearTimeout(globUpdateTimer)
		globUpdateTimer = window.setTimeout(doUpdateOverlays, 2000);
	}
	
	function initIcons() {
		for (var i = 20; i < 26; i++) {
			var icon = new GIcon();
			icon.image = "/static/marker/mark" + i + ".png";
			icon.shadow = "/static/marker/shadow.png";
			icon.iconSize = new GSize(25, 29);
			icon.shadowSize = new GSize(22, 20);
			icon.iconAnchor = new GPoint(13, 29);
			icon.infoWindowAnchor = new GPoint(13, 1);
			globIcons[i] = icon;
			
			globMarkerVisible[i] = true;
		}
	}
	
	function exportFile(format) {
		//alert(format);
		var bounds = getBounds();
		request = '';
		request += bounds.latA + ',' + bounds.lngA + ',' + 
				bounds.latB + ',' + bounds.lngB + ',' + bounds.zoom;
		for (var t in globTracks) {
			var track = globTracks[t];
			if (track.visible) {
				request += ',' + track.id;
			}	
		}
		window.location = '/data/export/' + request + '/export.' + format;
		return false;
	}
	
	function initGUI() {
		initIcons();
	
		$("#hide_link").click(function(){
			globMinimalized = !globMinimalized;
			if (globMinimalized) {
				$("#semiramis_content").animate({
					left: 0, top:0
				}, 
				globSpeed);
				$("#semiramis_data").slideUp(globSpeed);
				//$("#main_header").attr('src', "/static/map_logo_up.png");
				$("#main_header").css('background-image', "url(/static/gui/map_logo_up.png)");
			}
			else {
				$("#semiramis_content").animate({
					left: 20, top:20
				}, 
				globSpeed);
				$("#semiramis_data").slideDown(globSpeed);
				//$("#main_header").attr('src', "/static/map_logo_down.png");
				$("#main_header").css('background-image', "url(/static/gui/map_logo_down.png)");
			}
			
		});
			
		$("#home_link").click(function(){
			window.location = '/';
		});

		jQuery.Accordion.setDefaults({
			showSpeed: 300,
			hideSpeed: 300,
			active: false,
			alwaysOpen: false
		});
		jQuery("#nav").Accordion();
		
		$('.tooltipspan').Tooltip({
			track: true,
			delay: 0,
			showURL: true,
			showBody: "|"
		});
		
		var cookie = $.cookie('angleFormat');
		if (cookie != null) {
			globAngleFormat = cookie;
		}
	}
	
	
	$(document).ready(function() {
		initGUI();
		initMap();
	});	 
